/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "fixed.h"

fixed248_t fx248Fromi32(const int32_t b) {
  return (fixed248_t)b << FIXED248_FRACTION_BITS;
}

fixed248_t fx248Fromui32(const uint32_t b) {
  return (fixed248_t)b << FIXED248_FRACTION_BITS;
}

fixed248_t fx248Fromf32(const float_t b) {
  return (fixed248_t)(b * (1 << FIXED248_FRACTION_BITS));
}

int32_t fx248Toi32(const fixed248_t a) {
  return (int32_t)(a >> FIXED248_FRACTION_BITS);
}

uint32_t fx248Tou32(const fixed248_t a) {
  return (uint32_t)(a >> FIXED248_FRACTION_BITS);
}

float_t fx248Tof32(const fixed248_t a) {
  return (float_t)(a / (1 << FIXED248_FRACTION_BITS));
}

void fx248Addfx248(fixed248_t *a, const fixed248_t b) {
  *a += b;
}

void fx248Addi32(fixed248_t *a, const int32_t b) {
  *a += (fixed248_t)b << FIXED248_FRACTION_BITS;
}

void fx248Addui32(fixed248_t *a, const uint32_t b) {
  *a += (fixed248_t)b << FIXED248_FRACTION_BITS;
}

void fx248Subfx248(fixed248_t *a, const fixed248_t b) {
  *a -= b;
}

void fx248Subi32(fixed248_t *a, const int32_t b) {
  *a -= (fixed248_t)b << FIXED248_FRACTION_BITS;
}

void fx248Subui32(fixed248_t *a, const uint32_t b) {
  *a -= (fixed248_t)b << FIXED248_FRACTION_BITS;
}

void fx248Subf32(fixed248_t *a, const float_t b) {
  *a -= (fixed248_t)(b * (1 << FIXED248_FRACTION_BITS));
}

int32_t fx248Floor(const fixed248_t a) {
  return (int32_t)(a >> FIXED248_FRACTION_BITS);
}

int32_t fx248Ceil(const fixed248_t a) {
  return (int32_t)((
    a + ((1 << FIXED248_FRACTION_BITS) - 1)
  ) >> FIXED248_FRACTION_BITS);
}

int32_t fx248Round(const fixed248_t a) {
  return (int32_t)(
    (a + (1 << (FIXED248_FRACTION_BITS - 1))
  ) >> FIXED248_FRACTION_BITS);
}

uint32_t fx248Flooru32(const fixed248_t a) {
  return (uint32_t)(a >> FIXED248_FRACTION_BITS);
}

uint32_t fx248Ceilu32(const fixed248_t a) {
  return (uint32_t)((
    a + ((1 << FIXED248_FRACTION_BITS) - 1)
  ) >> FIXED248_FRACTION_BITS);
}

uint32_t fx248Roundu32(const fixed248_t a) {
  return (uint32_t)(
    (a + (1 << (FIXED248_FRACTION_BITS - 1))
  ) >> FIXED248_FRACTION_BITS);
}